/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlBinop.h"
#include "HvSignalLine.h"
#include "HvControlCast.h"
#include "HvSignalVar.h"
#include "HvMath.h"
#include "HvControlVar.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_3shJDE1C_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Ug0nDmLI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_c9HA7tvt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YCevMV4l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_VHEjSL5A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_EWKnuuey_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QKToVf2q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_iPbl3QYs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hY4fWOtR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_93sc97WO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_k70vv2tP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_ORM7rr12_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_fy10QrQD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YmrpoVlw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_YiKQOcZD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_23fJjWit_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_n8RpDSo0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vYhZCE2q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_aw6o0AWA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_dhFnKX5o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_PmdthVg6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xjtevuv6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_NwrQJjyy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_xZLK9aKd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_5maB32QX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ludnl7EB_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_zgC2G9x5;
  SignalLine sLine_yW66Gai9;
  SignalLine sLine_2AqBWBrN;
  ControlBinop cBinop_3shJDE1C;
  ControlBinop cBinop_Ug0nDmLI;
  ControlBinop cBinop_c9HA7tvt;
  ControlBinop cBinop_YCevMV4l;
  ControlBinop cBinop_QKToVf2q;
  ControlBinop cBinop_hY4fWOtR;
  ControlBinop cBinop_93sc97WO;
  ControlBinop cBinop_k70vv2tP;
  ControlVar cVar_ORM7rr12;
  ControlVar cVar_fy10QrQD;
  SignalVarf sVarf_OFAdzGsL;
  SignalVarf sVarf_W8536vDq;
  SignalVarf sVarf_cHzeVcLg;
  SignalVarf sVarf_8jKGsLQV;
  ControlBinop cBinop_vYhZCE2q;
  ControlBinop cBinop_xjtevuv6;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
